Struct CacheGlobalLock
pub struct CacheGlobalLock { /* private fields */ }Expand description
A global lock for the entire package cache.
This can be used to reduce lock overhead when performing many package operations by acquiring a single global lock instead of individual per-package locks.
Implementations
impl CacheGlobalLock
impl CacheGlobalLock
pub async fn acquire(
path: &Path,
) -> Result<CacheGlobalLock, PackageCacheLayerError>
pub async fn acquire( path: &Path, ) -> Result<CacheGlobalLock, PackageCacheLayerError>
Acquires a global write lock on the package cache.
This lock should be used to coordinate access across multiple package operations to reduce the overhead of acquiring individual locks.